/*******************************************************************************
 * JMMC project ( http://www.jmmc.fr ) - Copyright (C) CNRS.
 ******************************************************************************/

/***************************************************************************/

func amdlibUV(void)
/* DOCUMENT amdlibUV
    
  DESCRIPTION
    AMBER file manipulation scripts

  VERSION
    $Revision: 1.33 $
    
  FUNCTIONS
    - amdlibComputUvCoord        : Computes UV coordinates with an AMBER fits
                                   file given as input.
    - amdlibComputeBaseVect      : Computes ground baselines of the
                                   VLTI from a given ESO raw data file, or
                                   from a list of stations names.
    - amdlibGetUVCoordinates     : Compute UV coordinates with a series of
                                   AMBER fits files given as input.
    - amdlibGetVLTIBaseVect      : Gets the geographical coordinates based
                                   on VLTI stations.
    - amdlibPlotUVCoordinates    : This function plots UV coordinates.
    - amdlibPlotUsedVLTIStations : This function plots the used VLTI stations
    - amdlibUV                   : 
                                   corresponding to an observation.

    SEE ALSO
    yocoFile, amdlib
    */
{
    version = strpart(strtok("$Revision: 1.33 $",":")(2),2:-2);
    if (am_subroutine())
    {
        help, amdlibUV;
    }   
    return version;
}

/***************************************************************************/

struct amdlibSTARVIS
{
    string date;
    double u;
    double v;
    double w;
    double ra;
    double dec;
    double lst;
    double ha;
    double theta;
    double base;
    double delay; 
}

/***************************************************************************/

struct amdlibLOCATION
{
    string name;
    double tz;
    double dst;
    double lon;
    double lat;
    double elev;
}

/***************************************************************************/

func _amdlibGetVLTILocation(&loc)
/* DOCUMENT _amdlibGetVLTILocation(&loc)

  DESCRIPTION
    This function fills in the input amdlibLOCATION structure fields with the
    location on VLTI.

  PARAMETERS
    - loc : created amdlibLOCATION structure.

  EXAMPLES
    > _amdlibGetVLTILocation(l)
    You are here :  ESO, Cerro Paranal
    > l
    amdlibLOCATION(name="ESO, Cerro Paranal",tz=4,dst=-1,lon=-4.69365,
    lat=-24.6279,elev=2635)

  SEE ALSO
*/
{
    loc = amdlibLOCATION(0);
    loc.name = "ESO, Cerro Paranal";
    loc.tz = 4;
    loc.dst = -1;
    loc.lon = -4.69365;
    loc.lat = -24.62794830;
    loc.elev = 2635;
    /* Test output */
    yocoLogTest, swrite( format="You are here :\t%s\n", loc.name);
}

/***************************************************************************/

func _amdlibComputeUvwCoord(&data, bvect)
/* DOCUMENT _amdlibComputeUvwCoord(&data, bvect)

  DESCRIPTION
    Corrects uvw coordinates using base vector of observation bvect, and update
    data fields with results.

  PARAMETERS
    - data  : amdlibSTARVIS structure to be corrected.
    - bvect : base vector of observation.

  SEE ALSO
*/
{
    data.ha  = data.lst - data.ra;

    degr = 180 / pi;
    hour = degr / 15.;
    Bnorm = sqrt((bvect^2)(sum));

    /* Baseline vector in alt-az coordinates */
    Balt = asin(bvect(3)/ Bnorm) * degr;
    Baz  = atan(bvect(2), bvect(1)) * degr;

    /* Baseline vector in equatorial coordinates */
    Bdec = asin(sin(Balt/degr) * sin(loc.lat/degr) + 
                cos(Balt/degr) * cos(loc.lat/degr) * cos(Baz/degr)) * degr;

    yBha = sin(Balt/degr) * cos(loc.lat/degr) - 
        cos(Balt/degr) * cos(Baz/degr) * sin(loc.lat/degr);
    zBha = -1. * cos(Balt/degr) * sin(Baz/degr);
    Bha = (atan(zBha,yBha) * hour + 24) % 24;

    /* baseline vector in the equatorial cartesian frame */
    Lx = - (-Bnorm * cos(Bdec/degr) * cos(Bha/hour));
    Ly = - Bnorm * cos(Bdec/degr) * sin(Bha/hour);
    Lz = Bnorm * sin(Bdec/degr);

    /* projection of the baseline vector on the u,v,w frame */
    data.u = (sin(data.ha/hour) * Lx + cos(data.ha/hour) * Ly);
    data.v = (- sin(data.dec/degr) * cos(data.ha/hour) * Lx 
              + sin(data.dec/degr) * sin(data.ha/hour) * Ly
              + cos(data.dec/degr) * Lz);
    data.w = (cos(data.dec/degr) * cos(data.ha/hour) * Lx 
              - cos(data.dec/degr) * sin(data.ha/hour) * Ly
              + sin(data.dec/degr) * Lz);

    data.theta = atan(data.u, data.v) * degr;
    data.base  = sqrt(data.u^2 + data.v^2);
    data.delay = - data.w;
}

/***************************************************************************/ 

func _amdlibSaveVLTI_stations(void)
/* DOCUMENT _amdlibSaveVLTI_stations
       
  DESCRIPTION
    Saves a table containing the VLTI stations coordinates in observatory
    coordinates (P & Q) and in E-W, N-S coordinates (metres).
    
  RESULTS
    the file ~/.VLTI_positions.dat is written
    
  CAUTIONS
    This data comes from the ESO webpage
    http://www.eso.org/observing/etc/doc/vlti/baseline/vltistations.html
    which may not contain the most up-to-date positions of the stations.
    Therefore, an error can occur due to the obsolescence of this table.
    
  SEE ALSO
*/
{
    positions = " ID       P          Q          E          N    \n\
                 A0    -32.001    -48.013    -14.642    -55.812\n\
                 A1    -32.001    -64.021     -9.434    -70.949\n\
                 B0    -23.991    -48.019     -7.065    -53.212\n\
                 B1    -23.991    -64.011     -1.863    -68.334\n\
                 B2    -23.991    -72.011      0.739    -75.899\n\
                 B3    -23.991    -80.029      3.348    -83.481\n\
                 B4    -23.991    -88.013      5.945    -91.030\n\
                 B5    -23.991    -96.012      8.547    -98.594\n\
                 C0    -16.002    -48.013      0.487    -50.607\n\
                 C1    -16.002    -64.011      5.691    -65.735\n\
                 C2    -16.002    -72.019      8.296    -73.307\n\
                 C3    -16.002    -80.010     10.896    -80.864\n\
                 D0      0.010    -48.012     15.628    -45.397\n\
                 D1      0.010    -80.015     26.039    -75.660\n\
                 D2      0.010    -96.012     31.243    -90.787\n\
                 E0     16.011    -48.016     30.760    -40.196\n\
                 G0     32.017    -48.0172    45.896    -34.990\n\
                 G1     32.020   -112.010     66.716    -95.501\n\
                 G2     31.995    -24.003     38.063    -12.289\n\
                 H0     64.015    -48.007     76.150    -24.572\n\
                 I1     72.001    -87.997     96.711    -59.789\n\
                 J1     88.016    -71.992    106.648    -39.444\n\
                 J2     88.016    -96.005    114.460    -62.151\n\
                 J3     88.016      7.996     80.628     36.193\n\
                 J4     88.016     23.993     75.424     51.320\n\
                 J5     88.016     47.987     67.618     74.009\n\
                 J6     88.016     71.990     59.810     96.706\n\
                 K0     96.002    -48.006    106.397    -14.165\n\
                 L0    104.021    -47.998    113.977    -11.549\n\
                 M0    112.013    -48.000    121.535     -8.951\n\
                 U1    -16.000    -16.000     -9.925    -20.335\n\
                 U2     24.000     24.000     14.887     30.502\n\
                 U3     64.0013    47.9725    44.915     66.183\n\
                 U4    112.000      8.000    103.306     43.999";

    /* Saved station */
    fh = open("~/.VLTI_positions.dat", "w");
    write, fh, positions;
    close, fh;

    /* Verbose output */
    yocoLogInfo,"VLTI station saved into file","~/.VLTI_positions.dat";
}

/***************************************************************************/ 

func amdlibGetVLTIBaseVect(station1, station2, &orig)
/* DOCUMENT amdlibGetVLTIBaseVect(station1, station2, &orig)
   
  DESCRIPTION
    Gets the geographical coordinates based on VLTI stations.

  PARAMETERS
    - station1 : first station for considered baseline
    - station2 : second station for considered baseline
    - orig     : geagraphical origin coordinates.

  SEE ALSO
*/
{
  if(!open("~/.VLTI_positions.dat", "r",1))
    _amdlibSaveVLTI_stations;

    //xy legends ajusting
    adjust = [0.025,0.025];

    fh = open("~/.VLTI_positions.dat", "r");
    pos = "";
    BP = BQ = BE = BN = 0.0;
    read, fh, format="%s", pos;
    while (read(fh, format="%s %f %f %f %f", pos, BP, BQ, BE, BN))
    {
        if (pos == station1)
        {
            BE1 = BE;
            BN1 = BN;
            if (strmatch(station1, "U"))
            {
                BZ1 = 8.504;
            }
            else
            {
                BZ1 = 0;
            }
            orig = [BE1, BN1, BZ1]
                }
        else if (pos == station2)
        {
            BE2 = BE;
            BN2 = BN;
            if (strmatch(station2, "U"))
            {
                BZ2 = 8.504;
            }
            else
            {
                BZ2 = 0;
            }
        }
    }
    close, fh;

    /* Test output */
    yocoLogTest, swrite(format= station1 + "-" + station2 + " // base =%fm, angle=%fdeg",
                        sqrt((BE2-BE1)^2 + (BN2-BN1)^2),
                        180 / pi * atan(BN2-BN1, BE2-BE1) );
//     write, station1 + "-" + station2, " // base =", 
//         sqrt((BE2-BE1)^2 + (BN2-BN1)^2),
//         "m, angle =", 180 / pi * atan(BN2-BN1, BE2-BE1), "";

    return [BN2-BN1, BE2-BE1, BZ2-BZ1];
}

/************************************************************************/ 

func _amdlibPlotVLTIStations(color=)
/* DOCUMENT _amdlibPlotVLTIStations(color=)

  DESCRIPTION
    Plots different VLTI stations.

  PARAMETERS 
    - color : color function for plot.

  SEE ALSO
*/
{
  if(!open("~/.VLTI_positions.dat", "r",1))
    _amdlibSaveVLTI_stations;

    fh = open("~/.VLTI_positions.dat", "r");
    pos = "";
    BP = BQ = BE = BN = 0.0;
    read, fh, format="%s %f %f %f %f", pos, BP, BQ, BE, BN;

    nPointsPerCircle = 100;
    param = span(-pi, pi, nPointsPerCircle);

    while (read(fh, format="%s %f %f %f %f", pos, BP, BQ, BE, BN))
    {
        if (strmatch(pos, "U"))
        {
            diam = 8;
            diam2 = 26;
            just = "CH";
        }
        else
        {
            diam = 1.8;
            just = "RT";
        }

        if(!is_void(diam2))
            plg, diam2/2.*sin(param) + BN, diam2/2.*cos(param) + BE, color=[128,128,128],marks=0;
            
        plg, diam/2.*sin(param) + BN, diam/2.*cos(param) + BE, color=color,marks=0;
        plt, pos, BE, BN, tosys=1, height=6, justify=just, color=color;
    }

    limits, -53, 157, -105, 105;
}

/************************************************************************/     
      
func amdlibComputeBaseVect(&orig, &stations, &baseNames, fileName=)
/* DOCUMENT amdlibComputeBaseVect(&orig, &stations, &baseNames, fileName=)

  DESCRIPTION
    Computes ground baselines of the VLTI from a given ESO raw data file,
    or from a list of stations names.
    
  PARAMETERS
    - orig      : geagraphical telescop origin coordinates.
    - stations  : telescopes stations for considered baseline
    - baseNames : returned base names using the stations names
    - fileName  : name of file where stations information are extracted.

  RETURN VALUES
    ground baselines vectors.
    

  SEE ALSO
*/
{
    keys = ["HIERARCH ESO ISS CONF STATION1",
            "HIERARCH ESO ISS CONF STATION2",
            "HIERARCH ESO ISS CONF STATION3"];

    if (is_void(stations))
    {
        message = "FILE NAME\nEnter a file";
        if (amdlibFileChooser(message,fileName, 3) == 0)
        {
            yocoError,"'fileName' is not specified correctly.";
            return 0;
        }

        valuesTab = _amdlibGetKwdVals(fileName, keys,gui=0);

        stations = [valuesTab(1),valuesTab(2),valuesTab(3)];
        stations = stations(where(stations!="-"));
    }
    if (is_void(stations))
    {
        yocoGuiInfoBox, "failing to get AMBER stations information";
        return 0;
    }
    nbTels = numberof(stations);
    nbBases = nbTels * (nbTels - 1) / 2;

    B = baseNames = orig = [];
        
    //Determination du vecteur de base de l'observation
    for (iBase=1; iBase <= nbBases; iBase++)
    {
        st1 = [1,
               2,1,
               1,2,3,
               1,2,3,4,
               1,2,3,4,5,
               1,2,3,4,5,6,
               1,2,3,4,5,6,7];
        st2 = [2,
               3,3,
               4,4,4,
               5,5,5,5,
               6,6,6,6,6,
               7,7,7,7,7,7,
               8,8,8,8,8,8,8];

        grow, B, array(amdlibGetVLTIBaseVect(stations(st1(iBase)),
                                             stations(st2(iBase)), origP), 1);
        grow,baseNames,stations(st1(iBase)) + stations(st2(iBase))
            grow, orig, array(origP, 1);
    }

    return B;
}

/************************************************************************/

func amdlibComputUvCoord(&orig, &B, &u, &v, &w, &stations, file=)
/* DOCUMENT amdlibComputUvCoord(&orig, &B, &u, &v, &w, &stations, file=)

  DESCRIPTION
    This function computes UV coordinates with an AMBER fits
    file given as input. You can then compare them with the UV
    coordinates of the file header

  PARAMETERS
    - orig     : geagraphical telescop origin coordinates.
    - B        : projected baselines vector.
    - u        : Fourier space coordinates.
    - v        : Fourier space coordinates.
    - w        : Fourier space coordinates.
    - stations : VLTI stations
    - file     : input file.

  RETURN VALUES
    1 on succesful completion, 0 otherwise

  SEE ALSO
    amdlibComputeBaseVect, _amdlibComputeUvwCoord
*/
{
    stardata = amdlibSTARVIS(0);

    // Select cerro paranal
    _amdlibGetVLTILocation, loc;

    keys = ["DATE-OBS", "RA", "DEC", "LST"];

    message = "FILE NAME\nEnter a file";
    if (amdlibFileChooser(message,file,3) == 0)
    {
        yocoError,"'file' is not specified correctly";
        return 0;
    }

    valuesTab = _amdlibGetKwdVals(file, keys,gui=0);

    if (valuesTab(1) == "-")
    {
        yocoGuiInfoBox, "Not enough information in the file header";
        return 0;
    }

    dateObs = valuesTab(1);
    dateObsStr = yocoStrSplit(dateObs, "T");
    dateStr = dateObsStr(1);
    timsStr = dateObsStr(2);
    time = yocoStr2Double(yocoStrSplit(timsStr, ":"));

    date  = yocoStrSplit(dateStr, "-");
    day   = date(3);
    month = date(2);
    year  = date(1);
    stardata.date = day + "/" + month + "/" + year;

    stardata.lst = yocoStr2Double(valuesTab(4)) / 3600;

    // Detemine observed star's coordinates
    RA = yocoStr2Double(valuesTab(2));
    stardata.ra = RA/15.0;

    DEC = yocoStr2Double(valuesTab(3));
    stardata.dec = DEC;

    /* Test output */
    yocoLogTest, "You observed at " + timsStr + " " + stardata.date;

    // Determine observation's base vector
    B = amdlibComputeBaseVect(fileName=file, orig, stations);

    if (is_void(B))
    {
        /* In this case, information necessary for computing UV coordinates are
         * not present in primary header. So nothing is written, and no graphs
         * relative to uv coordinates have to be plot. */
        return 0;
    }
    nbBases = numberof(B(1, ));
    for (iBase=1; iBase <= nbBases; iBase++)
    {
        bvect = B(,iBase);

        //Calcul du vecteur de base dans l'espace reciproque
        _amdlibComputeUvwCoord, stardata, bvect;

        if (iBase == 1)
        {
            u = array(stardata.u, 1);
            v = array(stardata.v, 1);
            w = array(stardata.w, 1);
            theta = array(stardata.theta, 1);
            base = array(stardata.base, 1);
        }
        else
        {
            grow, u, stardata.u;
            grow, v, stardata.v;
            grow, w, stardata.w;
            grow, theta, stardata.theta;
            grow, base, stardata.base;

        }
    }

    /* Test output */
    yocoLogTest, swrite(format="u: %f  v: %f  w: %f  (m)", u,v,w);
    yocoLogTest, swrite(format="projected base length: %fm, angle: deg", base, theta);
    
    //     write, "u :", u, "m, v :", v, "m, w :", w, "m";
    //     write, "projected baseline :", base,
    //         "m, projected angle :", theta, "";
    
    return 1; 
}

/***************************************************************************/

func amdlibGetUVCoordinates(&w, fileName=)
/* DOCUMENT amdlibGetUVCoordinates(fileName=)

  DESCRIPTION
    Compute UV coordinates with an AMBER fits file given as input.

  PARAMETERS
    - fileName : name of file where uv coordinates have to be extracted.

  RETURN VALUES
    The uv coordinates array.

  SEE ALSO
*/
{
    nFiles = numberof(fileName);

    uvTable = [];

    for (iFile=1; iFile <= numberof(fileName); iFile++)
    {
        if (!amdlibComputUvCoord(orig, B, u, v, w, stations, 
                                 file=fileName(iFile)))
        {
            yocoGuiInfoBox, "failing to process AMBER UV information";
            return 0;
        }
        grow, uvTable, [u,v,w];
    }

    return transpose(uvTable, [1,2]);
}

/***************************************************************************/

func amdlibPlotUVCoordinates(&uvTable, fileName=, figDir=, nameString=, noLabels=, noTitle=, baseNames=, color=, stations=, width=)
/* DOCUMENT amdlibPlotUVCoordinates(&uvTable, fileName=, figDir=, nameString=, noLabels=, noTitle=, baseNames=, color=, stations=)

  DESCRIPTION
    This function plots UV coordinates.

  PARAMETERS
    - uvTable    : array containing corrected uv coordinates for each frame.
    - fileName   : name of observation file. Useful if no uvTable has 
                   already been computed. 
    - figDir     : name of the directory where results are saved as ps 
                   files. If not specified, graphs are not saved.
    - nameString : name of the bserved star.
    - noLabels   : if set to 0, labels are not plotted.
    - noTitle    : if set to 0, title are not plotted.
    - baseNames  : names for each baseline
    - color : color of the plot
    - stations : input stations if not present in the file name
    
  RETURN VALUES
    Returns 1 on sucessful completion, 0 otherwise.

  SEE ALSO
    plg
*/
{
    colors=["red","green","blue","cyan","magenta","yellow","black"];
    
    if (is_void(noTitle))
    {
        noTitle = 0;
    }
    if (is_void(noLabels))
    {
        noLabels = 0;
    }
        
    if (is_void(uvTable))
    {
        message = "FILE NAME\nEnter a file";
        if (amdlibFileChooser(message, fileName, 3) == 0)
        {
            yocoError,"'fileName' is not specified correctly";
            return 0;
        }
    }
        
    if (is_void(nameString))
    {
        titleString1 = "VLTI BASELINES";
        titleString2 = "VLTI PROJECTED BASELINES";
    }
    else
    {
        titleString1 = "VLTI BASELINES\n" + nameString;
        titleString2 = "VLTI PROJECTED BASELINES\n" + nameString;
    }

    nbFiles = numberof(fileName);

    if (nbFiles == 0)
    {
        if (is_void(uvTable))
        {
            yocoGuiInfoBox, "No files to get UV coordinates";
            return 0;
        }
        else
            nbFiles = dimsof(uvTable)(4);
    }
    
    if (is_void(uvTable))
    {
        for (iFile=1; iFile <= nbFiles; iFile++)
        {
            orig = B = u = v = w = stations = [];
            if (!amdlibComputUvCoord(orig, B, u, v, w, stations,
                                     file=fileName(iFile)))
            {
                yocoGuiInfoBox, "failing to process AMBER UV information";
                return 0;
            }
            
            grow, uvTable, array(transpose([u,v]),1);
        }
        
        baseNames = [];
        nbBases = dimsof(uvTable)(3);
        if(nbBases==1)
            nbTel = 2;
        else if(nbBases==3)
            nbTel = 3;
        tels = indgen(nbTel);
        for (iBase = 1; iBase <= nbBases; iBase++)
        {
            telUsed = int(roll(tels,1-iBase));
            if(is_void(stations))
                grow, baseNames, "";
            else
                grow, baseNames, pr1(iBase)+" = "+
                    stations(telUsed(1))+"-"+stations(telUsed(2));
        }

    }
    
    nPointsPerCircle = 30;
    param = span(-pi, pi, nPointsPerCircle);
    nbBases = dimsof(uvTable)(3);

    // (U,V) plot itself
    for (iFile=1; iFile <= nbFiles; iFile++)
    {
        for (iBase=1; iBase <= nbBases; iBase++)
        {
            telUsed = int(roll(tels,1-iBase));
            if (!strmatch(stations(telUsed(1)), "U"))
            {
                diam = 1.8;
            }
            else
            {
                diam = 8;
            }
            u = uvTable(1, ..);
            v = uvTable(2, ..);

            kolor = colors(iBase%7);
            if(!is_void(color))
                kolor = color;
            
            plg, diam / 2 * sin(param) + v(iBase,iFile), 
                diam / 2 * cos(param) + u(iBase,iFile), 
                color=kolor, width=width;
            plg, diam / 2 * sin(param) - v(iBase,iFile), 
                diam / 2 * cos(param) - u(iBase,iFile),
                color=kolor, width=width;
            plg, [0,0],  [0,0], marks=1, marker='\2',type="none";

            tmax = grow(v + diam / 2, u + diam / 2);
            tmin = grow(v - diam / 2, u - diam / 2);

            mx = max(max(tmax), -min(tmin))*1.1;
        }
    }

    limits, mx, -mx, -mx, mx;

    // Labellize the graph
    if(noLabels==0)
    {
        for (iBase=1; iBase <= nbBases; iBase++)
        {
            Base = sqrt(u(iBase,avg)^2 + v(iBase,avg)^2);
            Theta = 180/pi*atan(u(iBase,avg),v(iBase,avg));
            
            if(!is_void(baseNames))
                text1 = " "+baseNames(iBase);
            else
                text1 = " "+pr1(iBase);
            plt,  text1+ " : " + 
                strpart(pr1(Base), 1:5) + 
                "m, "+ strpart(pr1(Theta),1:5) + "",
                mx, mx - mx/9 * iBase, 
                tosys=1, color=colors(iBase%7);
            
            xytitles, "E <----- U (m)", "V (m) -----> N",[0.02,0.02];
        }
    }
    if(noTitle==0)
    {
        pltitle, titleString2;
    }

    // Hard copy
    if (!is_void(figDir))
    {  
        fileHeader = getGoodName(nameString);
        hcps, figDir + "/" + fileHeader + "_VLTI_proj_baselines.ps";
        yocoLogInfo, "Saved the 2 VLTI graphs";
    }

    return 1; 
}

/***************************************************************************/

func amdlibPlotUsedVLTIStations(&uvTable, &stations, fileName=, figDir=, nameString=, noLabels=, noTitle=, color=, off=, type=)
/* DOCUMENT amdlibPlotUsedVLTIStations(&uvTable, &stations, fileName=, figDir=, nameString=, noLabels=, noTitle=, color=, off=, type=)

  DESCRIPTION
    This function plots the used VLTI stations corresponding to
    an observation.

  PARAMETERS
    - uvTable    : array containing corrected uv coordinates for each frame.
    - stations   : 
    - fileName   : name of observation file. Useful if no uvTable has 
                   already been computed. 
    - figDir     : name of the directory where results are saved as ps
                   files. If not specified, graphs are not saved.
    - nameString : name of the bserved star.
    - noLabels   : if set to 0, labels are not plotted.
    - noTitle    : if set to 0, title are not plotted.
    - color : optional color of the plot
    - off : optional offset (in meters)
    - type : optional line type

  RETURN VALUES
    Returns 1 on sucessful completion, 0 otherwise.

  SEE ALSO
*/
{
    colors=["red","green","blue","cyan","magenta","yellow","black"];
    
    if (is_void(noTitle))
    {
        noTitle =0;
    }
    if (is_void(noLabels))
    {
        noLabels = 0;
    }
    if (is_void(off))
    {
        // offset of plot
        off = 0;
    }
    if (is_void(uvTable))
    {
        message = "FILE NAME\nEnter a file";
        if (amdlibFileChooser(message, fileName, 3) == 0)
        {
            yocoError, "'fileName' is not specified correctly.";
            return 0;
        }
    }
    
    if (is_void(nameString))
    {
        titleString1 = "VLTI BASELINES";
        titleString2 = "VLTI PROJECTED BASELINES";
    }
    else
    {
        titleString1 = "VLTI BASELINES\n" + nameString;
        titleString2 = "VLTI PROJECTED BASELINES\n" + nameString;
    }

    nbFiles = numberof(fileName);

    if (nbFiles == 0)
    {
        if (is_void(uvTable))
        {
            yocoGuiInfoBox, "No files to get UV coordinates";
            return 0;
        }
    }

    _amdlibPlotVLTIStations, color="black";
    
    if (is_void(stations))
    {
        B = amdlibComputeBaseVect(orig, stations,
                                  fileName=fileName(1));
    }
    else
    {
        B = amdlibComputeBaseVect(orig, stations);
    }
    nbTels = numberof(stations);
    nbBases = nbTels*(nbTels-1)/2;
    for (iBase=1; iBase <= nbBases; iBase++)
    {
        kolor = colors(iBase%7);
        if(!is_void(color))
            kolor = color;
        
        pldj, orig(1, iBase)+off, orig(2, iBase)+off,
            orig(1, iBase) + B(2, iBase)+off, 
            orig(2, iBase) + B(1, iBase)+off, 
            color=kolor, type=type;
    }
    
    // Labelling the graph
    if (noLabels == 0)
    {
        xytitles, "position (m) -----> E",
            "position (m) -----> N",
            [0.02,0.02];
        for (iBase=1; iBase <= nbBases; iBase++)
        {
            kolor = colors(iBase%7);
            if(!is_void(color))
                kolor = color;
            
            plt, "Base " + pr1(iBase), 
                0.185, 0.415 + 0.0155 * (nbBases - iBase+1), 
                tosys=0, color=kolor;
        }
    }
    if (noTitle == 0)
    {
        pltitle, titleString1;
    }
    if (NO_CRITE == 0)
    {
        amdlibCopyright;
    }

    // Hard copy
    if (!is_void(figDir))
    {  
        fileHeader = getGoodName(nameString);
        hcps, figDir + "/" + fileHeader + "_VLTI_baselines.ps";
        yocoLogInfo, "Saved the 2 VLTI graphs";
    }
    
    return 1; 
}

/***************************************************************************/
